<?php
/*
 *
 * PDF/X-3 starter:
 * Create PDF/X-3 conforming output
 *
 * Required software: PDFlib/PDFlib+PDI/PPS 10
 * Required data: font file, image file, ICC profile
 *                (see www.pdflib.com for output intent ICC profiles)
 */

/* This is where the data files are. Adjust as necessary.*/
$searchpath = dirname(__FILE__,3)."/input";
$imagefile = "nesrin.jpg";
$outfilename = "";

try {
    $p = new PDFlib();

    // This means we must check return values of load_font() etc.
    $p->set_option("errorpolicy=return");

    $p->set_option("searchpath={" . $searchpath . "}");

    if ($p->begin_document($outfilename, "pdfx=PDF/X-3:2003") == 0) {
        echo("Error: " . $p->get_errmsg());
        exit(1);
    }

    $p->set_info("Creator", "PDFlib starter sample");
    $p->set_info("Title", "starter_pdfx3");

    /* Load output intent ICC profile */
    if ($p->load_iccprofile("ISOcoated_v2_eci.icc", "usage=outputintent") == 0) {
        printf("Error: %s\n", $p->get_errmsg());
        printf("See www.pdflib.com for output intent ICC profiles.\n");
        $p = 0;
        exit(2);
    }

    $p->begin_page_ext(0, 0, "width=a4.width height=a4.height");

    $font = $p->load_font("NotoSerif-Regular", "unicode", "");
    if ($font == 0) {
        echo("Error: " . $p->get_errmsg());
        exit(1);
    }
    $p->setfont($font, 24);

    $spot = $p->makespotcolor("PANTONE 123 C");
    $p->setcolor("fill", "spot", $spot, 1.0, 0.0, 0.0);
    $p->fit_textline("PDF/X-3:2003 starter", 50, 700, "");

    $image = $p->load_image("auto", $imagefile, "");

    if ($image == 0) {
        echo("Error: " . $p->get_errmsg());
        exit(1);
    }

    $p->fit_image($image, 0.0, 0.0, "scale=0.5");

    $p->end_page_ext("");

    $p->end_document("");

    $buf = $p->get_buffer();
    $len = strlen($buf);

    header("Content-type: application/pdf");
    header("Content-Length: $len");
    header("Content-Disposition: inline; filename=starter_pdfx3.pdf");
    print $buf;

}
catch (PDFlibException $e) {
    echo("PDFlib exception occurred in starter_pdfx3 sample:\n" .
        "[" . $e->get_errnum() . "] " . $e->get_apiname() . ": " .
        $e->get_errmsg() . "\n");
    exit(1);
}
catch (Throwable $e) {
    echo($e);
    exit(1);
}

$p = 0;
?>
